%ASTRID class definition
%
%   Usage:
%      cluster=generic();
%      cluster=generic('np',3);
%      cluster=generic('np',3,'name','astrid','login','username');

classdef generic
    properties (SetAccess=public) 
		 % {{{1
		 name='generic';
		 login='';
		 np=feature('numcores');
		 port=0;
		 interactive=1;
		 codepath=[issmtier() '/bin'];
		 executionpath=[issmdir() '/execution'];
		 valgrind=[issmtier() '/externalpackages/valgrind/install/bin/valgrind'];
		 valgrindlib=[issmtier() '/externalpackages/valgrind/install/lib/libmpidebug.so'];
		 valgrindsup=[issmtier() '/externalpackages/valgrind/issm.supp'];
		 %}}}
	 end
	 methods
		 function cluster=generic(varargin) % {{{1
		 
			 %initialize cluster using user settings if provided
			 cluster.name=oshostname();
			 if (exist([cluster.name '_settings'])==2), generic_settings; end

			 %use provided options to change fields
			 options=pairoptions(varargin{:});
			 for i=1:size(options.list,1),
				 fieldname=options.list{i,1};
				 fieldvalue=options.list{i,2};
				 if ismember(fieldname,properties(generic)),
					 cluster.(fieldname)=fieldvalue;
				 else
					 disp(['''' fieldname ''' is not a property of cluster generic']);
				 end
			 end
		 end
		 %}}}
		 function disp(cluster) % {{{1
			 %  display the object
			 disp(sprintf('class ''%s'' object ''%s'' = ',class(cluster),inputname(1)));
			 disp(sprintf('    name: %s',cluster.name));
			 disp(sprintf('    login: %s',cluster.login));
			 disp(sprintf('    np: %i',cluster.np));
			 disp(sprintf('    port: %i',cluster.port));
			 disp(sprintf('    codepath: %s',cluster.codepath));
			 disp(sprintf('    executionpath: %s',cluster.executionpath));
			 disp(sprintf('    valgrind: %s',cluster.valgrind));
			 disp(sprintf('    valgrindlib: %s',cluster.valgrindlib));
			 disp(sprintf('    valgrindsup: %s',cluster.valgrindsup));
		 end
		 %}}}
		 function IsConsistent(cluster) % {{{1
			 if cluster.np>feature('numcores'),
				 error(['number of processors exceeds number of cores seen by matlab ' num2str(feature('numcores'))]);
			 end
			 if isnan(cluster.np),
				 error('number of processors should not be NaN!');
			 end
		 end
		 %}}}
		 function BuildQueueScript(cluster,md) % {{{1
		 
			 %retrieve parameters
			 modelname=md.name;
			 solution_type=md.solution_type;
			 mem_debug=md.mem_debug;

			 %open file for writing: 
			 fid=fopen([modelname '.queue'],'w');

			 %write instructions for launching a job on the cluster
			 fprintf(fid,'#!/bin/sh\n');
			 if mem_debug==0,
				 if cluster.interactive
					 fprintf(fid,'mpiexec -np %i %s/issm.exe %s %s %s.bin %s.petsc %s.outbin %s.lock ',...
						 cluster.np,cluster.codepath,EnumToString(solution_type),cluster.executionpath,modelname,modelname,modelname,modelname);
				 else
					 fprintf(fid,'mpiexec -np %i %s/issm.exe %s %s %s.bin %s.petsc %s.outbin %s.lock  2> %s.errlog >%s.outlog ',...
						 cluster.np,cluster.codepath,EnumToString(solution_type),cluster.executionpath,modelname,modelname,modelname,modelname,modelname,modelname);
				 end
			 else
				 %fprintf(fid,'LD_PRELOAD=%s mpiexec -np %i %s --leak-check=full --gen-suppressions=all --suppressions=%s %s/issm.exe %s %s %s.bin %s.petsc %s.outbin %s.lock  2> %s.errlog >%s.outlog ',cluster.valgrindlib,cluster.np,cluster.valgrind,cluster.valgrindsup,cluster.codepath,EnumToString(solution_type),cluster.executionpath,modelname,modelname,modelname,modelname,modelname,modelname);
				 fprintf(fid,'LD_PRELOAD=%s mpiexec -np %i %s --leak-check=full --suppressions=%s %s/issm.exe %s %s %s.bin %s.petsc %s.outbin %s.lock  2> %s.errlog >%s.outlog ',...
					 cluster.valgrindlib,cluster.np,cluster.valgrind,cluster.valgrindsup, cluster.codepath,EnumToString(solution_type),cluster.executionpath,modelname,modelname,modelname,modelname,modelname,modelname);
			 end

			 if md.gprof,
				 fprintf(fid,'\n gprof %s/issm.exe gmon.out > %s.performance',cluster.codepath,modelname);
			 end

			 if ~md.io_gather,
				 %concatenate the output files:
				 fprintf(fid,'\ncat %s.outbin.* > %s.outbin',modelname,modelname);
			 end

			 %in interactive mode, create a run file, and errlog and outlog file
			 if cluster.interactive,
				 fid=fopen([modelname '.errlog'],'w'); fclose(fid);
				 fid=fopen([modelname '.outlog'],'w'); fclose(fid);
			 end

		 end
		 %}}}
		 function LaunchQueueJob(cluster,md,options)% {{{1
			 
			 %lauch command, to be executed via ssh
			 launchcommand=['cd ' cluster.executionpath ' && rm -rf ./' md.runtimename ' && mkdir ' md.runtimename ...
			                ' && cd ' md.runtimename ' && mv ../' md.runtimename '.tar.gz ./ && tar -zxf ' md.runtimename '.tar.gz  && source  ' md.name '.queue '];

			if ~strcmpi(options.batch,'yes'),
				
				%compress the files into one zip.
				compressstring=['tar -zcf ' md.runtimename '.tar.gz ' md.name '.bin ' md.name '.queue '  md.name '.petsc '];
				if md.qmu_analysis,
					compressstring=[compressstring md.name '.qmu.in'];
				end
				if cluster.interactive,
					compressstring=[compressstring ' ' md.name '.errlog ' md.name '.outlog '];
				end
				system(compressstring);
				
				disp('uploading input file and queueing script');
				issmscpout(cluster.name,cluster.executionpath,cluster.login,cluster.port,{[md.runtimename '.tar.gz']});

				disp('launching solution sequence on remote cluster');
				issmssh(cluster.name,cluster.login,cluster.port,launchcommand);
			else
				disp('batch mode requested: not launching job interactively');
				disp('launch solution sequence on remote cluster by hand');
			end

		 end %}}}
		 function Download(cluster,md)% {{{1

			%some check
			if isempty(md.runtimename),
				error('supply runtime name for results to be loaded!');
			end

			%Figure out the  directory where all the files are in: 
			directory=[cluster.executionpath '/' md.runtimename '/'];

			%What packages are we picking up from remote cluster
			packages={[md.name '.outlog'],[md.name '.errlog']};
			if md.qmu_analysis,
				packages{end+1}=[md.name '.qmu.err'];
				packages{end+1}=[md.name '.qmu.out'];
				if isfield(md.qmu_params,'tabular_graphics_data'),
					if md.qmu_params.tabular_graphics_data==true,
						packages{end+1}='dakota_tabular.dat'; 
					end
				end
			else
				packages{end+1}=[md.name '.outbin'];
			end

			%copy files from cluster to present directory
			issmscpin(cluster.name, cluster.login, cluster.port, directory, packages);
		end %}}}
	end
end
